/*******************************************************************************
 * Copyright (c) 2003, 2007 s IT Solutions AT Spardat GmbH .
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     s IT Solutions AT Spardat GmbH - initial API and implementation
 *******************************************************************************/

/*
 * 
 * Created on 25.06.2003
 * 
 */
package at.spardat.xma.security;

import java.util.HashMap;
import java.util.Map;

import javax.security.auth.Subject;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;

import at.spardat.xma.plugins.PluginManager;


/**
 * xma specific login context. This class is used for client side login
 * instead of the JAAS standard class javax.security.auth.login.LoginContext.
 * This substitution is neccessary to be able to use the xma plugin mechanism
 * for configuration an loading of LoginModules.
 * 
 * XMALoginContext uses LoginModules with are complient to the JAAS-Standard.
 * 
 * @author s2877, s3595
 */
public class XMALoginContext {
    Subject subject;
    LoginModule module;

    /**
     * Creates a XMALoginContext, loads the configured LoginModule using the 
     * xma plugin mechanism and initializes the LoginModule. 
     * 
     * @param   handler         handler for callbacks
     * @param   manager         plugin manager to lookup module
     * @throws  LoginException  on login failures
     */
    public XMALoginContext(CallbackHandler handler, PluginManager manager, Map options ) throws LoginException {
        subject = new Subject();
        module = (LoginModule)manager.getPlugin( LoginModule.class );

        module.initialize(subject,handler,new HashMap(),options);
    }
    
    /**
     * Does a client side login using its LoginModule.
     * @throws LoginException  on login failures
     */
    public void login() throws LoginException {
        if(module.login()) {
            module.commit();
        } else {
            module.abort();
            throw new LoginException("login failed");
        }
    }
    
    /**
     * Does a client side logout using its LoginModule.
     * @throws LoginException on logout failures
     */
    public void logout() throws LoginException {
        boolean success = module.logout();
        if(!success) throw new LoginException("logout failed");
    }
    
    /**
     * Gets the authenticated Subject. The subject will only contain
     * user information after a sucessfull login.
     * @return Subject the subject
     */
    public Subject getSubject() {
        return subject;
    }
    
    /** Gets the used login module. */
    public final LoginModule getModule() {
        return module;
    }
}
